@@ -1,20 +1,19 @@
-#' Separate one column into multiple columns.
+#' Separate a character column into multiple columns with a regular
+#' expression or numeric locations
 #'
-#' Given either regular expression or a vector of character positions,
+#' Given either a regular expression or a vector of character positions,
 #' `separate()` turns a single character column into multiple columns.
 #'
 #' @inheritParams extract
-#' @param into Names of new variables to create as character vector.
-#'    Use `NA` to omit the variable in the output.
 #' @param sep Separator between columns.
 #'
-#'   If character, is interpreted as a regular expression. The default
+#'   If character, `sep` is interpreted as a regular expression. The default
 #'   value is a regular expression that matches any sequence of
 #'   non-alphanumeric values.
 #'
-#'   If numeric, interpreted as positions to split at. Positive values start
-#'   at 1 at the far-left of the string; negative value start at -1 at the
-#'   far-right of the string. The length of `sep` should be one less than
+#'   If numeric, `sep` is interpreted as character positions to split at. Positive
+#'   values start at 1 at the far-left of the string; negative value start at -1 at
+#'   the far-right of the string. The length of `sep` should be one less than
 #'   `into`.
 #' @param extra If `sep` is a character vector, this controls what
 #'   happens when there are too many pieces. There are three valid options:
@@ -28,16 +27,12 @@
 #'   * "warn" (the default): emit a warning and fill from the right
 #'   * "right": fill with missing values on the right
 #'   * "left": fill with missing values on the left
-#' @param remove If `TRUE`, remove input column from output data frame.
-#' @param convert If `TRUE`, will run [type.convert()] with
-#'   `as.is = TRUE` on new columns. This is useful if the component
-#'   columns are integer, numeric or logical.
-#' @param ... Additional arguments passed on to methods.
 #' @seealso [unite()], the complement, [extract()] which uses regular
 #'   expression capturing groups.
 #' @export
 #' @examples
 #' library(dplyr)
+#' # If you want to split by any non-alphanumeric value (the default):
 #' df <- data.frame(x = c(NA, "a.b", "a.d", "b.c"))
 #' df %>% separate(x, c("A", "B"))
 #'
@@ -45,56 +40,75 @@
 #' df %>% separate(x, c(NA, "B"))
 #'
 #' # If every row doesn't split into the same number of pieces, use
-#' # the extra and fill arguments to control what happens
+#' # the extra and fill arguments to control what happens:
 #' df <- data.frame(x = c("a", "a b", "a b c", NA))
 #' df %>% separate(x, c("a", "b"))
-#' # The same behaviour but no warnings
+#' # The same behaviour as previous, but drops the c without warnings:
 #' df %>% separate(x, c("a", "b"), extra = "drop", fill = "right")
-#' # Another option:
+#' # Opposite of previous, keeping the c and filling left:
 #' df %>% separate(x, c("a", "b"), extra = "merge", fill = "left")
+#' # Or you can keep all three:
+#' df %>% separate(x, c("a", "b", "c"))
 #'
-#' # If only want to split specified number of times use extra = "merge"
+#' # To only split a specified number of times use extra = "merge":
 #' df <- data.frame(x = c("x: 123", "y: error: 7"))
 #' df %>% separate(x, c("key", "value"), ": ", extra = "merge")
+#'
+#' # Use regular expressions to separate on multiple characters:
+#' df <- data.frame(x = c(NA, "a?b", "a.d", "b:c"))
+#' df %>% separate(x, c("A","B"), sep = "([.?:])")
+#'
+#' # convert = TRUE detects column classes:
+#' df <- data.frame(x = c("a:1", "a:2", "c:4", "d", NA))
+#' df %>% separate(x, c("key","value"), ":") %>% str
+#' df %>% separate(x, c("key","value"), ":", convert = TRUE) %>% str
 separate <- function(data, col, into, sep = "[^[:alnum:]]+", remove = TRUE,
                      convert = FALSE, extra = "warn", fill = "warn", ...) {
+  ellipsis::check_dots_used()
   UseMethod("separate")
 }
 #' @export
 separate.data.frame <- function(data, col, into, sep = "[^[:alnum:]]+",
                                 remove = TRUE, convert = FALSE,
                                 extra = "warn", fill = "warn", ...) {
-  orig <- data
-
+  check_present(col)
   var <- tidyselect::vars_pull(names(data), !! enquo(col))
   value <- as.character(data[[var]])
 
+  new_cols <- str_separate(value,
+    into = into,
+    sep = sep,
+    convert = convert,
+    extra = extra,
+    fill = fill
+  )
+  out <- append_df(data, new_cols, var, remove = remove)
+  reconstruct_tibble(data, out, if (remove) var else NULL)
+}
+
+str_separate <- function(x, into, sep, convert = FALSE, extra = "warn", fill = "warn") {
   if (!is.character(into)) {
     abort("`into` must be a character vector")
   }
 
   if (is.numeric(sep)) {
-    l <- strsep(value, sep)
+    out <- strsep(x, sep)
   } else if (is_character(sep)) {
-    l <- str_split_fixed(value, sep, length(into), extra = extra, fill = fill)
+    out <- str_split_fixed(x, sep, length(into), extra = extra, fill = fill)
   } else {
     abort("`sep` must be either numeric or character")
   }
 
-  names(l) <- as_utf8_character(into)
-  l <- l[!is.na(names(l))]
+  names(out) <- as_utf8_character(into)
+  out <- out[!is.na(names(out))]
   if (convert) {
-    l[] <- map(l, type.convert, as.is = TRUE)
+    out[] <- map(out, type.convert, as.is = TRUE)
   }
-
-  # Insert into existing data frame
-  data <- append_df(data, l, var, remove = remove)
-
-  reconstruct_tibble(orig, data, if (remove) var else NULL)
+  as_tibble(out)
 }
 
 strsep <- function(x, sep) {
-  nchar <- stringi::stri_length(x)
+  nchar <- nchar(x)
   pos <- map(sep, function(i) {
     if (i >= 0) return(i)
     pmax(0, nchar + i)
@@ -102,9 +116,10 @@ strsep <- function(x, sep) {
   pos <- c(list(0), pos, list(nchar))
 
   map(1:(length(pos) - 1), function(i) {
-    stringi::stri_sub(x, pos[[i]] + 1, pos[[i + 1]])
+    substr(x, pos[[i]] + 1, pos[[i + 1]])
   })
 }
+
 str_split_fixed <- function(value, sep, n, extra = "warn", fill = "warn") {
   if (extra == "error") {
     warn(glue(
@@ -118,7 +133,7 @@ str_split_fixed <- function(value, sep, n, extra = "warn", fill = "warn") {
   fill <- arg_match(fill, c("warn", "left", "right"))
 
   n_max <- if (extra == "merge") n else -1L
-  pieces <- stringi::stri_split_regex(value, sep, n_max)
+  pieces <- str_split_n(value, sep, n_max = n_max)
 
   simp <- simplifyPieces(pieces, n, fill == "left")
 
@@ -137,6 +152,24 @@ str_split_fixed <- function(value, sep, n, extra = "warn", fill = "warn") {
   simp$strings
 }
 
+str_split_n <- function(x, pattern, n_max = -1) {
+  m <- gregexpr(pattern, x, perl = TRUE)
+  if (n_max > 0) {
+    m <- lapply(m, function(x) slice_match(x, seq_along(x) < n_max))
+  }
+  regmatches(x, m, invert = TRUE)
+}
+
+slice_match <- function(x, i) {
+  structure(
+    x[i],
+    match.length = attr(x, "match.length")[i],
+    index.type = attr(x, "index.type"),
+    useBytes = attr(x, "useBytes")
+  )
+}
+
+
 list_indices <- function(x, max = 20) {
   if (length(x) > max) {
     x <- c(x[seq_len(max)], "...")
